# -*- coding: utf-8 -*-

# The parametrize function is generated, so this doesn't work:
#
#     from pytest.mark import parametrize
#
import pytest
parametrize = pytest.mark.parametrize

from btsprice.exchanges import Exchanges
from btsprice.bts_price_after_match import BTSPriceAfterMatch
from btsprice.bts_price_after_match import get_median
from pprint import pprint


class TestMain(object):
    logfile = open("/tmp/test-btsprice.log", 'a')

    def test_get_median(self):
        assert get_median([1, 2, 3, 4]) == 2.5
        assert get_median([1, 2, 3]) == 2
        assert get_median([1]) == 1
        assert get_median([]) is None

    def test_exchanges_yahoo(self):
        exchanges = Exchanges()
        rate_cny = exchanges.fetch_from_yahoo()
        pprint("======= test_exchanges_yahoo =========", self.logfile)
        pprint(rate_cny, self.logfile)
        assert rate_cny["USD"] > 0

    def test_exchanges_yunbi(self):
        exchanges = Exchanges()
        order_book = exchanges.fetch_from_yunbi()
        pprint("======= test_exchanges_yunbi =========", self.logfile)
        pprint(order_book, self.logfile)
        assert len(order_book["bids"]) > 0

    def test_exchanges_poloniex(self):
        exchanges = Exchanges()
        order_book = exchanges.fetch_from_poloniex()
        pprint("======= test_exchanges_poloniex =========", self.logfile)
        pprint(order_book, self.logfile)
        assert len(order_book["bids"]) > 0

    def test_exchanges_btc38(self):
        exchanges = Exchanges()
        order_book = exchanges.fetch_from_btc38()
        pprint("======= test_exchanges_btc38 =========", self.logfile)
        pprint(order_book, self.logfile)
        assert len(order_book["bids"]) > 0

    def test_exchanges_bter(self):
        exchanges = Exchanges()
        order_book = exchanges.fetch_from_bter()
        pprint("======= test_exchanges_bter =========", self.logfile)
        pprint(order_book, self.logfile)
        assert len(order_book["bids"]) > 0

    def test_bts_price_after_match(self):
        bts_price = BTSPriceAfterMatch()
        bts_price.set_need_cover(False)
        bts_price.get_rate_from_yahoo()

        # get all order book
        bts_price.get_order_book_all()

        # can add weight here
        for order_type in bts_price.order_types:
            for _order in bts_price.order_book["btc38_btc"][order_type]:
                _order[1] *= 0.1

        # calculate real price
        volume, volume2, real_price = bts_price.get_real_price(spread=0.01)
        valid_depth = bts_price.get_valid_depth(price=real_price, spread=0.01)
        pprint("======= test_bts_price_after_match =========", self.logfile)
        pprint([volume, real_price,
                real_price/1.01, real_price/0.99], self.logfile)
        pprint(valid_depth, self.logfile)
        # pprint(bts_price.order_book, self.logfile)
        assert volume > 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

from btsprice.bts_price_after_match import BTSPriceAfterMatch
import time
import logging
import logging.handlers
import os
from prettytable import PrettyTable
import locale
locale.setlocale(locale.LC_ALL, '')


class FeedPrice(object):

    def __init__(self, config=None):
        self.init_config(config)
        self.init_bts_price()
        self.setup_log()
        self.init_mpa_info()

    def init_config(self, config):
        if config:
            self.config = config
            return
        config = {}
        config["timer"] = 60
        config["max_update_hours"] = 23.5
        config["price_limit"] = {
            "change_min": 0.5, "change_max": 50, "spread": 0.01,
            "median_length": 9}
        config["market_weight"] = {
            "poloniex_btc": 1, "yunbi_cny": 1, "btc38_cny": 1, "btc38_btc": 1}

        self.config = config

    def init_bts_price(self):
        self.bts_price = BTSPriceAfterMatch()
        # don't use cover order, because it's feed price related,
        # if there is a big order, feed price will down without stop
        self.bts_price.set_need_cover(False)

    def setup_log(self):
        # Setting up Logger
        self.logger = logging.getLogger('bts')
        self.logger.setLevel(logging.INFO)
        formatter = logging.Formatter(
            '%(asctime)s[%(levelname)s]: %(message)s')
        fh = logging.handlers.RotatingFileHandler("/tmp/bts_delegate_task.log")
        fh.setFormatter(formatter)
        self.logger.addHandler(fh)

    def init_mpa_info(self):
        peg_asset_list = ["KRW", "BTC", "SILVER", "GOLD", "TRY",
                          "SGD", "HKD", "RUB", "SEK", "NZD", "CNY",
                          "MXN", "CAD", "CHF", "AUD", "GBP", "JPY",
                          "EUR", "USD", "SHENZHEN", "NASDAQC", "NIKKEI",
                          "HANGSENG", "SHANGHAI", "TCNY"]
        self.price_queue = {}
        for asset in peg_asset_list:
            self.price_queue[asset] = []
        self.time_publish_feed = 0
        self.adjust_scale = 1.00

    def fetch_bts_price(self):
        # updat rate cny
        self.bts_price.get_rate_from_yahoo()

        # get all order book
        self.bts_price.get_order_book_all()

        # can add weight here
        for order_type in self.bts_price.order_types:
            for market in self.bts_price.order_book:
                if market not in self.config["market_weight"]:
                    _weight = 0.0
                else:
                    _weight = self.config["market_weight"][market]
                for _order in self.bts_price.order_book[market][order_type]:
                    _order[1] *= _weight

        # calculate real price
        volume, volume_sum, real_price = self.bts_price.get_real_price(
            spread=self.config["price_limit"]["spread"])
        self.valid_depth = self.bts_price.get_valid_depth(
            price=real_price,
            spread=self.config["price_limit"]["spread"])
        price_cny = real_price / self.bts_price.rate_btc["CNY"]
        self.logger.info("fetch price is %.5f CNY/BTS, volume is %.3f",
                         price_cny, volume)
        self.logger.info("efficent depth : %s" % self.valid_depth)
        return real_price, volume

    # these MPA's precision is 100, it's too small,
    # have to change the price
    # but we should fixed these at BTS2.0
    def patch_nasdaqc(self, median_price):
        if "SHENZHEN" in median_price:
            median_price["SHENZHEN"] /= median_price["CNY"]
        if "SHANGHAI" in median_price:
            median_price["SHANGHAI"] /= median_price["CNY"]
        if "NASDAQC" in median_price:
            median_price["NASDAQC"] /= median_price["USD"]
        if "NIKKEI" in median_price:
            median_price["NIKKEI"] /= median_price["JPY"]
        if "HANGSENG" in median_price:
            median_price["HANGSENG"] /= median_price["HKD"]

    def handle_median_price(self, bts_price_in_btc):
        median_price = {}
        for asset in self.price_queue:
            if asset not in self.bts_price.rate_btc or \
                    self.bts_price.rate_btc[asset] is None:
                continue
            self.price_queue[asset].append(bts_price_in_btc
                                           / self.bts_price.rate_btc[asset])
            if len(self.price_queue[asset]) > \
                    self.config["price_limit"]["median_length"]:
                self.price_queue[asset].pop(0)
            median_price[asset] = sorted(
                self.price_queue[asset])[int(len(self.price_queue[asset]) / 2)]
        self.patch_nasdaqc(median_price)
        self.median_price = median_price

    def display_depth(self, volume):
        t = PrettyTable([
            "market", "bid price", "bid_volume", "ask price", "ask_volume"])
        t.align = 'r'
        t.border = True
        for market in sorted(self.valid_depth):
            _bid_price = "%.8f" % self.valid_depth[market]["bid_price"]
            _bid_volume = "{:,.0f}".format(
                self.valid_depth[market]["bid_volume"])
            _ask_price = "%.8f" % self.valid_depth[market]["ask_price"]
            _ask_volume = "{:,.0f}".format(
                self.valid_depth[market]["ask_volume"])
            t.add_row([
                market, _bid_price, _bid_volume, _ask_price, _ask_volume])
        print(t.get_string())

    def display_price(self):
        t = PrettyTable([
            "asset", "current(/BTC)", "current(/BTS)", "current(BTS/)",
            "median(/BTS)", "median(BTS/)"])
        t.align = 'r'
        t.border = True
        for asset in sorted(self.median_price):
            _price_btc = "%.3f" % (1/self.bts_price.rate_btc[asset])
            _price_bts1 = "%.8f" % self.price_queue[asset][-1]
            _price_bts2 = "%.3f" % (1/self.price_queue[asset][-1])
            _median_bts1 = "%.8f" % self.median_price[asset]
            _median_bts2 = "%.3f" % (1/self.median_price[asset])
            t.add_row([
                asset, _price_btc, _price_bts1,
                _price_bts2, _median_bts1, _median_bts2])
        print(t.get_string())

    def task_feed_price(self):
        bts_price_in_btc, volume = self.fetch_bts_price()
        self.handle_median_price(bts_price_in_btc)
        os.system("clear")
        cur_t = time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(time.time()))
        bts_price_in_cny = bts_price_in_btc / self.bts_price.rate_btc["CNY"]
        print("[%s] efficent price: %.5f CNY/BTS, depth: %s BTS" % (
            cur_t, bts_price_in_cny, "{:,.0f}".format(volume)))
        self.display_depth(volume)
        print()
        self.display_price()

    def excute(self):
        while True:
            try:
                self.task_feed_price()
            except Exception as e:
                self.logger.exception(e)
            time.sleep(int(self.config["timer"]))

if __name__ == '__main__':
    feedprice = FeedPrice()
    feedprice.excute()



# -*- coding: utf-8 -*-
import requests
import json
import logging
from btsprice.yahoo import Yahoo


class Exchanges():
    # ------------------------------------------------------------------------
    # Init
    # ------------------------------------------------------------------------
    def __init__(self):
        self.header = {'content-type': 'application/json',
                       'User-Agent': 'Mozilla/5.0 Gecko/20100101 Firefox/22.0'}
        self.log = logging.getLogger('bts')
        self.order_types = ["bids", "asks"]
        self.yahoo = Yahoo()

    # ------------------------------------------------------------------------
    # Fetch data
    # ------------------------------------------------------------------------
    #
    def fetch_from_btc38(self, quote="cny", base="bts"):
        try:
            url = "http://api.btc38.com/v1/depth.php"
            params = {'c': base, 'mk_type': quote}
            response = requests.get(
                url=url, params=params, headers=self.header, timeout=3)
            result = json.loads(vars(response)['_content'].decode("utf-8-sig"))
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            return {"bids": order_book_bid, "asks": order_book_ask}
        except:
            self.log.error("Error fetching results from btc38!")
            return

    def fetch_from_bter(self, quote="cny", base="bts"):
        try:
            url = "http://data.bter.com/api/1/depth/%s_%s" % (base, quote)
            result = requests.get(
                url=url, headers=self.header, timeout=3).json()
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            for bid_order in result["bids"]:
                order_book_bid.append(
                    [float(bid_order[0]), float(bid_order[1])])
            return {"bids": order_book_bid, "asks": order_book_ask}
        except:
            self.log.error("Error fetching results from bter!")
            return

    def fetch_from_yunbi(self, quote="cny", base="bts"):
        try:
            url = "https://yunbi.com/api/v2/depth.json"
            params = {'market': base+quote}
            result = requests.get(
                url=url, params=params, headers=self.header, timeout=3).json()
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            return {"bids": order_book_bid, "asks": order_book_ask}
        except:
            self.log.error("Error fetching results from yunbi!")
            return

    def fetch_from_poloniex(self, quote="btc", base="bts"):
        try:
            quote = quote.upper()
            base = base.upper()
            url = "http://poloniex.com/public?command=\
returnOrderBook&currencyPair=%s_%s" % (quote, base)

            result = requests.get(
                url=url, headers=self.header, timeout=3).json()
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            for bid_order in result["bids"]:
                order_book_bid.append(
                    [float(bid_order[0]), float(bid_order[1])])
            return {"bids": order_book_bid, "asks": order_book_ask}
        except:
            self.log.error("Error fetching results from bter!")
            return

    def fetch_from_yahoo(self, assets=None):
        return(self.yahoo.fetch_price())
